part of http;

/// 内容类型
abstract class ContentType {
  /// text/plain; charset=utf-8
  static ContentType get text => ContentType('text', 'plain', 'UTF-8');

  /// text/html; charset=utf-8
  static ContentType get html => ContentType('text', 'html', 'UTF-8');

  /// application/json; charset=utf-8
  static ContentType get json => ContentType('application', 'json', 'UTF-8');

  /// application/octet-stream
  static ContentType get binary => ContentType('application', 'octet-stream');

  /// MIMEType
  String get mimeType;

  /// PrimaryType
  String get primaryType;

  /// SubType
  String get subType;

  /// Charset
  String get charset;

  /// 构造函数
  factory ContentType(String primaryType, String subType, [String charset]) =>
      _ContentType(primaryType, subType, charset);
}

class _ContentType implements ContentType {
  final String _primaryType;
  final String _subType;
  final String _charset;

  _ContentType(this._primaryType, this._subType, this._charset);

  @override
  String get mimeType => '${this.primaryType}/${this.subType}';
  @override
  String get primaryType => this._primaryType;
  @override
  String get subType => this._subType;
  @override
  String get charset => this._charset;
}

/// 扩展方法
extension Extension on ContentType {
  /// 获取请求头
  HTTPHeader get header {
    final key = 'content-type';
    final value = this.charset == null
        ? this.mimeType
        : '${this.mimeType};charset=${this.charset}';
    return HTTPHeader(key, value);
  }
}
