part of http;

/// 使用方便的 HTTPClient 类
abstract class HTTPClient {
  /// 使用 GET 方式发送请求
  Future<HTTPResponse> getAsync(String uri, [List<HTTPHeader> headers]);

  /// 使用 POST 方式发送请求
  Future<HTTPResponse> postAsync(
      String uri, Object content, ContentType contentType,
      [List<HTTPHeader> headers]);

  /// 释放资源
  void close();

  /// 构造函数
  factory HTTPClient() => _HTTPClient();
}

class _HTTPClient implements HTTPClient {
  final Client _client;

  _HTTPClient() : _client = Client();

  @override
  Future<HTTPResponse> getAsync(String uri, [List<HTTPHeader> headers]) async {
    headers = headers ?? [];
    final map = Map<String, String>.fromIterable(headers,
        key: (e) => e.key, value: (e) => e.value);
    final response = await _client.get(uri, headers: map);
    return HTTPResponse(response.statusCode, response.body);
  }

  @override
  Future<HTTPResponse> postAsync(
      String uri, Object content, ContentType contentType,
      [List<HTTPHeader> headers]) async {
    contentType = contentType ?? ContentType.text;
    headers = headers ?? [];
    final header = contentType.header;
    headers.removeWhere((e) => e.key.toLowerCase() == header.key);
    headers.add(header);
    final map = Map<String, String>.fromIterable(headers,
        key: (e) => e.key, value: (e) => e.value);
    final response = await _client.post(uri, headers: map, body: content);
    return HTTPResponse(response.statusCode, response.body);
  }

  @override
  void close() {
    _client.close();
  }
}
